@@ -18,13 +18,13 @@ import errno
 import logging
 import os
 import requests
+import socket
 import sys
 import time
 from datetime import datetime
 from pytz import timezone
 from docker.errors import ImageNotFound
 from toil.lib.memoize import memoize
-from toil.lib.misc import mkdir_p
 from toil.lib.retry import retry
 from toil.version import currentCommit
 
@@ -415,7 +415,7 @@ def logProcessContext(config):
     # toil.version.version (string) cannot be imported at top level because it conflicts with
     # toil.version (module) and Sphinx doesn't like that.
     from toil.version import version
-    log.info("Running Toil version %s.", version)
+    log.info("Running Toil version %s on host %s.", version, socket.gethostname())
     log.debug("Configuration: %s", config.__dict__)
 
 
@@ -611,7 +611,7 @@ try:
                         if not os.path.exists(dir_path):
                             log.debug('Creating parent directory %s', dir_path)
                             # A race would be ok at this point
-                            mkdir_p(dir_path)
+                            os.makedirs(dir_path, exist_ok=True)
                     else:
                         raise
                 else:
