@@ -53,6 +53,8 @@ from toil.batchSystems.abstractBatchSystem import (AbstractBatchSystem,
                                                    EXIT_STATUS_UNAVAILABLE_VALUE,
                                                    UpdatedBatchJobInfo)
 from toil.common import Toil
+from toil.lib.bioio import configureRootLogger
+from toil.lib.bioio import setLogLevel
 from toil.lib.humanize import human2bytes
 from toil.lib.threading import LastProcessStandingArena
 from toil.resource import Resource
@@ -67,17 +69,22 @@ def retryable_kubernetes_errors(e):
     exceptions thrown by Kubernetes. 
     """
     if isinstance(e, urllib3.exceptions.MaxRetryError) or \
+       isinstance(e, urllib3.exceptions.ProtocolError) or \
         isinstance(e, ApiException):
         return True
     return False
-
-def retry_kubernetes(retry_while=retryable_kubernetes_errors):
+    
+def retryable_kubernetes_errors_expecting_gone(e):
     """
-    A wrapper that sends retryable Kubernetes predicates into a context-manager which will allow 
-    Kubernetes to keep retrying until a False or an executable method is seen.  
+    A function that determins whether or not Toil should retry or stop given 
+    exceptions thrown by Kubernetes, when Toil is expecting a 404 indicating
+    that a resource is gone, as it should be.
+    
+    Retries on errors other than 404.
     """
-    return retry(predicate=retry_while)
-
+    
+    return retryable_kubernetes_errors(e) and not (isinstance(e, ApiException) and e.status == 404)
+    
 def slow_down(seconds):
     """
     Toil jobs that have completed are not allowed to have taken 0 seconds, but
@@ -261,14 +268,72 @@ class KubernetesBatchSystem(BatchSystemLocalSupport):
 
         For example, calling self._api('batch').create_namespaced_job(self.namespace, job),
         Kubernetes can behave inconsistently and fail given a large job. See 
-        https://github.com/DataBiosphere/toil/issues/2884 .
+        https://github.com/DataBiosphere/toil/issues/2884.
         
         This function gives Kubernetes more time to try an executable api.  
         """
 
-        for attempt in retry_kubernetes():
+        for attempt in retry(predicate=retryable_kubernetes_errors):
             with attempt:
                 return method(*args, **kwargs)
+                
+    def _try_kubernetes_expecting_gone(self, method, *args, **kwargs):
+        """
+        Same as _try_kubernetes, but raises 404 errors as soon as they are
+        encountered (because we are waiting for them) instead of retrying on
+        them.
+        """
+        
+        for attempt in retry(predicate=retryable_kubernetes_errors_expecting_gone):
+            with attempt:
+                return method(*args, **kwargs)
+                
+    def _try_kubernetes_stream(self, method, *args, **kwargs):
+        """
+        Kubernetes kubernetes.watch.Watch().stream() streams can fail and raise
+        errors. We don't want to have those errors fail the entire workflow, so
+        we handle them here.
+        
+        When you want to stream the results of a Kubernetes API method, call
+        this instead of stream().
+        
+        To avoid having to do our own timeout logic, we finish the watch early
+        if it produces an error.
+        """
+    
+        w = kubernetes.watch.Watch()
+        
+        # We will set this to bypass our second catch in the case of user errors.
+        userError = False
+    
+        try: 
+            for item in w.stream(method, *args, **kwargs):
+                # For everything the watch stream gives us
+                try:
+                    # Show the item to user code
+                    yield item
+                except Exception as e:
+                    # If we get an error from user code, skip our catch around
+                    # the Kubernetes generator.
+                    userError = True
+                    raise
+        except Exception as e:
+            # If we get an error
+            if userError:
+                # It wasn't from the Kubernetes watch generator. Pass it along.
+                raise
+            else:
+                # It was from the Kubernetes watch generator we manage.
+                if retryable_kubernetes_errors(e):
+                    # This is just cloud weather.
+                    # TODO: We will also get an APIError if we just can't code good against Kubernetes. So make sure to warn.
+                    logger.warning("Received error from Kubernetes watch stream: %s", e)
+                    # Just end the watch.
+                    return
+                else:
+                    # Something actually weird is happening.
+                    raise
+            
 
     def setUserScript(self, userScript):
         logger.info('Setting user script for deployment: {}'.format(userScript))
@@ -632,11 +697,10 @@ class KubernetesBatchSystem(BatchSystemLocalSupport):
         if self.enableWatching:
             # Try watching for something to happen and use that.
 
-            w = kubernetes.watch.Watch()    
-
             if self.enableWatching:
                 for j in self._ourJobObjects():
-                    for event in w.stream(self._api('core').list_namespaced_pod, self.namespace, timeout_seconds=maxWait):
+                    for event in self._try_kubernetes_stream(self._api('core').list_namespaced_pod, self.namespace, timeout_seconds=maxWait):
+                        # For each event from the stream until it times out or just disconnects
                         pod = event['object']
                         if pod.metadata.name.startswith(self.jobPrefix):
                             if pod.status.phase == 'Failed' or pod.status.phase == 'Succeeded':
@@ -859,10 +923,10 @@ class KubernetesBatchSystem(BatchSystemLocalSupport):
         
         
         try:
-            # Delete the job and all dependents (pods)
-            self._try_kubernetes(self._api('batch').delete_namespaced_job, jobObject.metadata.name,
-                                                     self.namespace,
-                                                     propagation_policy='Foreground')
+            # Delete the job and all dependents (pods), hoping to get a 404 if it's magically gone
+            self._try_kubernetes_expecting_gone(self._api('batch').delete_namespaced_job, jobObject.metadata.name,
+                                                self.namespace,
+                                                propagation_policy='Foreground')
                                                 
             # That just kicks off the deletion process. Foreground doesn't
             # actually block. See
@@ -874,10 +938,11 @@ class KubernetesBatchSystem(BatchSystemLocalSupport):
             # non-existence.
             self._waitForJobDeath(jobObject.metadata.name)
                     
-        except kubernetes.client.rest.ApiException:
-            # TODO: check to see if this is a 404 on the thing we tried to delete
-            # If so, it is gone already and we don't need to delete it again.
-            pass
+        except ApiException as e:
+            if e.status != 404:
+                # Something is wrong, other than the job already being deleted.
+                raise
+            # Otherwise everything is fine and the job is gone. 
 
         # Return the one finished job we found
         return UpdatedBatchJobInfo(jobID=jobID, exitStatus=exitCode, wallTime=runtime, exitReason=None)
@@ -894,13 +959,17 @@ class KubernetesBatchSystem(BatchSystemLocalSupport):
         while True:
             try:
                 # Look for the job
-                self._try_kubernetes(self._api('batch').read_namespaced_job, jobName, self.namespace)
+                self._try_kubernetes_expecting_gone(self._api('batch').read_namespaced_job, jobName, self.namespace)
                 # If we didn't 404, wait a bit with exponential backoff
                 time.sleep(backoffTime)
                 if backoffTime < maxBackoffTime:
                     backoffTime *= 2
-            except kubernetes.client.rest.ApiException:
+            except ApiException as e:
                 # We finally got a failure!
+                if e.status != 404:
+                    # But it wasn't due to the job being gone; something is wrong.
+                    raise
+                # It was a 404; the job is gone. Stop polling it.
                 break
             
     def shutdown(self):
@@ -924,12 +993,14 @@ class KubernetesBatchSystem(BatchSystemLocalSupport):
             # Kill jobs whether they succeeded or failed
             try:
                 # Delete with background poilicy so we can quickly issue lots of commands
-                response = self._try_kubernetes(self._api('batch').delete_namespaced_job, jobName, 
-                                                                    self.namespace, 
-                                                                    propagation_policy='Background')
+                response = self._try_kubernetes_expecting_gone(self._api('batch').delete_namespaced_job, jobName, 
+                                                               self.namespace, 
+                                                               propagation_policy='Background')
                 logger.debug('Killed job for shutdown: %s', jobName)
             except ApiException as e:
-                logger.error("Exception when calling BatchV1Api->delte_namespaced_job: %s" % e)
+                if e.status != 404:
+                    # Anything other than a 404 is weird here.
+                    logger.error("Exception when calling BatchV1Api->delte_namespaced_job: %s" % e)
 
 
     def _getIssuedNonLocalBatchJobIDs(self):
@@ -1017,7 +1088,8 @@ def executor():
 
     """
 
-    logging.basicConfig(level=logging.DEBUG)
+    configureRootLogger()
+    setLogLevel("DEBUG")
     logger.debug("Starting executor")
     
     # If we don't manage to run the child, what should our exit code be?
