@@ -22,7 +22,7 @@ import logging
 import os
 import re
 import sys
-from toil import subprocess
+import subprocess
 import tempfile
 import time
 from threading import Thread
@@ -32,9 +32,9 @@ from six.moves.queue import Empty, Queue
 from six import itervalues
 
 from toil.lib.iterables import concat
-from toil import which
+from shutil import which
 
-from toil.batchSystems.abstractBatchSystem import BatchSystemSupport
+from toil.batchSystems.abstractBatchSystem import BatchSystemSupport, UpdatedBatchJobInfo
 from toil.lib.bioio import getTempFile
 from toil.common import Toil
 
@@ -57,7 +57,7 @@ class ParasolBatchSystem(BatchSystemSupport):
     def __init__(self, config, maxCores, maxMemory, maxDisk):
         super(ParasolBatchSystem, self).__init__(config, maxCores, maxMemory, maxDisk)
         if maxMemory != sys.maxsize:
-            logger.warn('The Parasol batch system does not support maxMemory.')
+            logger.warning('The Parasol batch system does not support maxMemory.')
         # Keep the name of the results file for the pstat2 command..
         command = config.parasolCommand
         if os.path.sep not in command:
@@ -118,16 +118,16 @@ class ParasolBatchSystem(BatchSystemSupport):
             stdout, stderr = process.communicate()
             status = process.wait()
             for line in stderr.decode('utf-8').split('\n'):
-                if line: logger.warn(line)
+                if line: logger.warning(line)
             if status == 0:
                 return 0, stdout.decode('utf-8').split('\n')
             message = 'Command %r failed with exit status %i' % (command, status)
             if autoRetry:
-                logger.warn(message)
+                logger.warning(message)
             else:
                 logger.error(message)
                 return status, None
-            logger.warn('Waiting for a 10s, before trying again')
+            logger.warning('Waiting for a 10s, before trying again')
             time.sleep(10)
 
     parasolOutputPattern = re.compile("your job ([0-9]+).*")
@@ -213,7 +213,7 @@ class ParasolBatchSystem(BatchSystemSupport):
             runningJobs = self.getIssuedBatchJobIDs()
             if set(jobIDs).difference(set(runningJobs)) == set(jobIDs):
                 break
-            logger.warn( 'Tried to kill some jobs, but something happened and they are still '
+            logger.warning( 'Tried to kill some jobs, but something happened and they are still '
                          'going, will try againin 5s.')
             time.sleep(5)
         # Update the CPU usage, because killed jobs aren't written to the results file.
@@ -269,16 +269,16 @@ class ParasolBatchSystem(BatchSystemSupport):
     def getUpdatedBatchJob(self, maxWait):
         while True:
             try:
-                jobID, status, wallTime = self.updatedJobsQueue.get(timeout=maxWait)
+                item = self.updatedJobsQueue.get(timeout=maxWait)
             except Empty:
                 return None
             try:
-                self.runningJobs.remove(jobID)
+                self.runningJobs.remove(item.jobID)
             except KeyError:
                 # We tried to kill this job, but it ended by itself instead, so skip it.
                 pass
             else:
-                return jobID, status, wallTime
+                return item
 
     def updatedJobWorker(self):
         """
@@ -341,10 +341,10 @@ class ParasolBatchSystem(BatchSystemSupport):
                             wallTime = float( max( 1, usrTicks + sysTicks) ) * 0.01
                         else:
                             wallTime = float(endTime - startTime)
-                        self.updatedJobsQueue.put((jobId, status, wallTime))
+                        self.updatedJobsQueue.put(UpdatedBatchJobInfo(jobID=jobId, exitStatus=status, wallTime=wallTime, exitReason=None))
                 time.sleep(1)
         except:
-            logger.warn("Error occurred while parsing parasol results files.")
+            logger.warning("Error occurred while parsing parasol results files.")
             raise
         finally:
             for fileHandle in resultsFileHandles:
@@ -356,11 +356,11 @@ class ParasolBatchSystem(BatchSystemSupport):
             exitValue = self._runParasol(['-results=' + results, 'clear', 'sick'],
                                          autoRetry=False)[0]
             if exitValue is not None:
-                logger.warn("Could not clear sick status of the parasol batch %s" % results)
+                logger.warning("Could not clear sick status of the parasol batch %s" % results)
             exitValue = self._runParasol(['-results=' + results, 'flushResults'],
                                          autoRetry=False)[0]
             if exitValue is not None:
-                logger.warn("Could not flush the parasol batch %s" % results)
+                logger.warning("Could not flush the parasol batch %s" % results)
         self.running = False
         logger.debug('Joining worker thread...')
         self.worker.join()
