@@ -18,7 +18,8 @@ import logging
 import tempfile
 import threading
 import time
-from toil import subprocess
+
+import subprocess
 import signal
 import os
 import errno
