@@ -19,18 +19,17 @@ from past.utils import old_div
 import logging
 import os
 from pipes import quote
-import subprocess
 import time
 import math
 import sys
 import shlex
 import xml.etree.ElementTree as ET
 import tempfile
+from toil.lib.misc import call_command, CalledProcessErrorStderr
 
 from toil.batchSystems import MemoryString
 from toil.batchSystems.abstractGridEngineBatchSystem import AbstractGridEngineBatchSystem, UpdatedBatchJobInfo
 
-logging.basicConfig(level=logging.DEBUG)
 logger = logging.getLogger(__name__)
 
 
@@ -48,20 +47,19 @@ class TorqueBatchSystem(AbstractGridEngineBatchSystem):
             """ Determines PBS/Torque version via pbsnodes
             """
             try:
-                out = subprocess.check_output(["pbsnodes", "--version"]).decode('utf-8')
-
+                out = call_command(["pbsnodes", "--version"])
                 if "PBSPro" in out:
                      logger.debug("PBS Pro proprietary Torque version detected")
                      self._version = "pro"
                 else:
                      logger.debug("Torque OSS version detected")
                      self._version = "oss"
-            except subprocess.CalledProcessError as e:
+            except CalledProcessErrorStderr as e:
                if e.returncode != 0:
                     logger.error("Could not determine PBS/Torque version")
 
             return self._version
-        
+
         """
         Torque-specific AbstractGridEngineWorker methods
         """
@@ -77,16 +75,13 @@ class TorqueBatchSystem(AbstractGridEngineBatchSystem):
             # PBS plain qstat will return every running job on the system.
             jobids = sorted(list(currentjobs.keys()))
             if self._version == "pro":
-                process = subprocess.Popen(['qstat', '-x'] + jobids, stdout=subprocess.PIPE)
+                stdout = call_command(['qstat', '-x'] + jobids)
             elif self._version == "oss":
-                process = subprocess.Popen(['qstat'] + jobids, stdout=subprocess.PIPE)
+                stdout = call_command(['qstat'] + jobids)
 
-
-            stdout, stderr = process.communicate()
-
-            # qstat supports XML output which is more comprehensive, but PBSPro does not support it 
+            # qstat supports XML output which is more comprehensive, but PBSPro does not support it
             # so instead we stick with plain commandline qstat tabular outputs
-            for currline in stdout.decode('utf-8').split('\n'):
+            for currline in stdout.split('\n'):
                 items = currline.strip().split()
                 if items:
                     jobid = items[0].strip()
@@ -120,15 +115,13 @@ class TorqueBatchSystem(AbstractGridEngineBatchSystem):
                 return UpdatedBatchJobInfo(jobID=jobID, exitStatus=retcode, wallTime=None, exitReason=None)
 
         def killJob(self, jobID):
-            subprocess.check_call(['qdel', self.getBatchSystemID(jobID)])
+            call_command(['qdel', self.getBatchSystemID(jobID)])
 
         def prepareSubmission(self, cpu, memory, jobID, command, jobName):
             return self.prepareQsub(cpu, memory, jobID) + [self.generateTorqueWrapper(command, jobID)]
 
         def submitJob(self, subLine):
-            process = subprocess.Popen(subLine, stdout=subprocess.PIPE)
-            so, se = process.communicate()
-            return so
+            return call_command(subLine)
 
         def getJobExitCode(self, torqueJobID):
             if self._version == "pro":
@@ -136,10 +129,9 @@ class TorqueBatchSystem(AbstractGridEngineBatchSystem):
             elif self._version == "oss":
                 args = ["qstat", "-f", str(torqueJobID).split('.')[0]]
 
-            process = subprocess.Popen(args, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
-            for line in process.stdout:
+            stdout = call_command(args)
+            for line in stdout.split('\n'):
                 line = line.strip()
-                #logger.debug("getJobExitCode exit status: " + line)
                 # Case differences due to PBSPro vs OSS Torque qstat outputs
                 if line.startswith("failed") or line.startswith("FAILED") and int(line.split()[1]) == 1:
                     return 1
@@ -187,10 +179,10 @@ class TorqueBatchSystem(AbstractGridEngineBatchSystem):
                     raise ValueError("Incompatible resource arguments ('mem=', 'nodes=', 'ppn='): {}".format(reqlineEnv))
 
                 reqline.append(reqlineEnv)
-            
+
             if reqline:
                 qsubline += ['-l',','.join(reqline)]
-            
+
             # All other qsub parameters can be passed through the environment (see man qsub).
             # No attempt is made to parse them out here and check that they do not conflict
             # with those that we already constructed above
@@ -221,7 +213,7 @@ class TorqueBatchSystem(AbstractGridEngineBatchSystem):
             fh.write(command + "\n")
 
             fh.close
-            
+
             return tmpFile
 
 
