@@ -68,20 +68,22 @@ from toil.fileStores.abstractFileStore import AbstractFileStore
 from cwltool.loghandler import _logger as cwllogger
 from cwltool.loghandler import defaultStreamHandler
 from cwltool.pathmapper import (PathMapper, adjustDirObjs, adjustFileObjs,
-                                get_listing, MapperEnt, visit_class,
-                                normalizeFilesDirs, downloadHttpFile)
+                                MapperEnt, visit_class,
+                                downloadHttpFile)
 from cwltool.process import (shortname, fill_in_defaults, compute_checksums,
                              add_sizes, Process)
 from cwltool.secrets import SecretStore
 from cwltool.software_requirements import (
     DependenciesConfiguration, get_container_from_software_requirements)
-from cwltool.utils import aslist, convert_pathsep_to_unix
+from cwltool.utils import aslist, convert_pathsep_to_unix, get_listing, normalizeFilesDirs, CWLObjectType
 from cwltool.mutation import MutationManager
 
 from toil.job import Job, Promise
 from toil.common import Config, Toil, addOptions
 from toil.version import baseVersion
 
+logger = logging.getLogger(__name__)
+
 # Define internal jobs we should avoid submitting to batch systems and logging
 CWL_INTERNAL_JOBS = ("CWLJobWrapper",
                      "CWLWorkflow",
@@ -131,7 +133,7 @@ def filter_skip_null(name: str, value: Any) -> Any:
     err_flag = [False]
     value = _filter_skip_null(value, err_flag)
     if err_flag[0]:
-        cwllogger.warning(
+        logger.warning(
             "In %s, SkipNull result found and cast to None. \n"
             "You had a conditional step that did not run, "
             "but you did not use pickValue to handle the skipped input." % name)
@@ -310,7 +312,7 @@ class ResolveSource:
             return values
 
         if not isinstance(values, list):
-            cwllogger.warning(
+            logger.warning(
                 "pickValue used but input %s is not a list." % self.name)
             return values
 
@@ -323,13 +325,13 @@ class ResolveSource:
             else:
                 return result[0]
 
-        elif pick_value_type == "only_non_null":
+        elif pick_value_type == "the_only_non_null":
             if len(result) == 0:
                 raise cwltool.errors.WorkflowException(
-                    "%s: only_non_null operator found no non-null values" % self.name)
+                    "%s: the_only_non_null operator found no non-null values" % self.name)
             elif len(result) > 1:
                 raise cwltool.errors.WorkflowException(
-                    "%s: only_non_null operator found more than one non-null values" % self.name)
+                    "%s: the_only_non_null operator found more than one non-null values" % self.name)
             else:
                 return result[0]
 
@@ -369,7 +371,7 @@ class StepValueFrom:
         self.context = self.source.resolve()
         return self.context
 
-    def do_eval(self, inputs: Dict[str, cwltool.expression.JSON]) -> Any:
+    def do_eval(self, inputs: CWLObjectType) -> Any:
         """
         Evaluate the valueFrom expression with the given input object
 
@@ -469,6 +471,10 @@ class ToilPathMapper(PathMapper):
         self.stage_listing = stage_listing
         super(ToilPathMapper, self).__init__(
             referenced_files, basedir, stagedir, separateDirs=separateDirs)
+            
+    def mapper(self, src: str) -> MapperEnt:
+        result = super(ToilPathMapper, self).mapper(src)
+        return result
 
     def visit(self,
               obj: Dict[str, Any],
@@ -476,6 +482,7 @@ class ToilPathMapper(PathMapper):
               basedir: str,
               copy: bool = False,
               staged: bool = False) -> None:
+              
         tgt = convert_pathsep_to_unix(os.path.join(stagedir, obj["basename"]))
         if obj["location"] in self._pathmap:
             return
@@ -489,6 +496,7 @@ class ToilPathMapper(PathMapper):
 
             if obj["location"].startswith("file://") and not self.stage_listing:
                 staged = False
+                
             self.visitlisting(
                 obj.get("listing", []), tgt, basedir, copy=copy, staged=staged)
 
@@ -507,7 +515,7 @@ class ToilPathMapper(PathMapper):
                     obj,
                     "location",
                     validate.ValidationException,
-                    cwllogger.isEnabledFor(logging.DEBUG),
+                    logger.isEnabledFor(logging.DEBUG),
                 ):
                     deref = self.get_file(path) if self.get_file else ab
                     if deref.startswith("file:"):
@@ -529,6 +537,7 @@ class ToilPathMapper(PathMapper):
                     self._pathmap[path] = MapperEnt(
                         deref, tgt, "WritableFile" if copy else "File", staged
                     )
+            
             self.visitlisting(
                 obj.get("secondaryFiles", []),
                 stagedir,
@@ -582,6 +591,7 @@ class ToilFsAccess(cwltool.stdfsaccess.StdFsAccess):
 
         Overwrites cwltool.stdfsaccess.StdFsAccess._abs() to account for toil specific schema.
         """
+        
         # Used to fetch a path to determine if a file exists in the inherited cwltool.stdfsaccess.StdFsAccess,
         # (among other things) so this should not error on missing files.
         # See: https://github.com/common-workflow-language/cwltool/blob/beab66d649dd3ee82a013322a5e830875e8556ba/cwltool/stdfsaccess.py#L43
@@ -627,10 +637,44 @@ def write_file(writeFunc: Any, index: dict, existing: dict, file_uri: str) -> st
                 index[file_uri] = "toilfs:" + writeFunc(rp).pack()
                 existing[index[file_uri]] = file_uri
             except Exception as e:
-                cwllogger.error("Got exception '%s' while copying '%s'", e, file_uri)
+                logger.error("Got exception '%s' while copying '%s'", e, file_uri)
                 raise
         return index[file_uri]
 
+def prepareDirectoryForUpload(directory_metadata: dict,
+                              skip_broken: bool = False) -> None:
+    """
+    Prepare a Directory object to be uploaded.
+
+    Assumes listings are already filled in.
+
+    Makes sure the directory actually exists, and rewrites its location to be
+    something we can use on another machine.
+    
+    Since Files and sub-Directories are already tracked by the directory's
+    listing, we just need some sentinel path to represent the existence of a
+    directory coming from Toil and not the local filesystem.
+    """
+    if directory_metadata["location"].startswith("toilfs:") or directory_metadata["location"].startswith("_:"):
+        # Already in Toil; nothing to do
+        return
+    if not directory_metadata["location"] and directory_metadata["path"]:
+        directory_metadata["location"] = schema_salad.ref_resolver.file_uri(directory_metadata["path"])
+    if directory_metadata["location"].startswith("file://") and not os.path.isdir(directory_metadata["location"][7:]):
+        if skip_broken:
+            return
+        else:
+            raise cwltool.errors.WorkflowException(
+                "Directory is missing: %s" % directory_metadata["location"])
+                
+    # The metadata for a directory is all we need to keep around for it. It
+    # doesn't have a real location. But each directory needs a unique location
+    # or cwltool won't ship the metadata along. cwltool takes "_:" as a signal
+    # to make directories instead of copying from somewhere. So we give every
+    # directory a unique _: location and cwltool's machinery Just Works.
+    directory_metadata["location"] = "_:" + directory_metadata["location"]
+    
+    logger.debug("Sending directory at %s", directory_metadata["location"])
 
 def uploadFile(uploadfunc: Any,
                fileindex: dict,
@@ -641,6 +685,7 @@ def uploadFile(uploadfunc: Any,
     Update a file object so that the location is a reference to the toil file
     store, writing it to the file store if necessary.
     """
+    
     if file_metadata["location"].startswith("toilfs:") or file_metadata["location"].startswith("_:"):
         return
     if file_metadata["location"] in fileindex:
@@ -656,6 +701,8 @@ def uploadFile(uploadfunc: Any,
                 "File is missing: %s" % file_metadata["location"])
     file_metadata["location"] = write_file(
         uploadfunc, fileindex, existing, file_metadata["location"])
+        
+    logger.debug("Sending file at: %s", file_metadata["location"])
 
 
 def writeGlobalFileWrapper(file_store: AbstractFileStore, fileuri: str) -> str:
@@ -687,6 +734,8 @@ def toilStageFiles(file_store: AbstractFileStore,
         if isinstance(obj, dict):
             if obj.get("class") in ("File", "Directory"):
                 yield obj
+                for dir_entry in _collectDirEntries(obj.get("secondaryFiles", [])):
+                    yield dir_entry
             else:
                 for sub_obj in obj.values():
                     for dir_entry in _collectDirEntries(sub_obj):
@@ -697,40 +746,32 @@ def toilStageFiles(file_store: AbstractFileStore,
                     yield dir_entry
 
     jobfiles = list(_collectDirEntries(cwljob))
-    pm = ToilPathMapper(
-        jobfiles, "", outdir, separateDirs=False, stage_listing=True)
+    pm = ToilPathMapper(jobfiles, "", outdir, separateDirs=False, stage_listing=True)
     for _, p in pm.items():
-        if not p.staged:
-            continue
+        if p.staged:
+            if destBucket:
+                # Directories don't need to be created if we're exporting to a bucket
+                if p.type == "File":
+                    # Remove the staging directory from the filepath and
+                    # from the destination URL
+                    unstageTargetPath = p.target[len(outdir):]
+                    destUrl = '/'.join(s.strip('/') for s in [destBucket, unstageTargetPath])
 
-        # Deal with bucket exports
-        if destBucket:
-            # Directories don't need to be created if we're exporting to
-            # a bucket
-            if p.type == "File":
-                # Remove the staging directory from the filepath and
-                # form the destination URL
-                unstageTargetPath = p.target[len(outdir):]
-                destUrl = '/'.join(s.strip('/')
-                                   for s in [destBucket, unstageTargetPath])
-
-                file_store.exportFile(FileID.unpack(p.resolved[7:]), destUrl)
-
-            continue
-
-        if not os.path.exists(os.path.dirname(p.target)):
-            os.makedirs(os.path.dirname(p.target), 0o0755)
-        if p.type == "File" and not os.path.exists(p.target):
-            file_store.exportFile(FileID.unpack(p.resolved[7:]), "file://" + p.target)
-        elif p.type == "Directory" and not os.path.exists(p.target):
-            os.makedirs(p.target, 0o0755)
-        elif p.type == "CreateFile":
-            with open(p.target, "wb") as n:
-                n.write(p.resolved.encode("utf-8"))
+                    file_store.exportFile(FileID.unpack(p.resolved[7:]), destUrl)
+            else:
+                if not os.path.exists(os.path.dirname(p.target)):
+                    os.makedirs(os.path.dirname(p.target), 0o0755)
+                if p.type == "File" and not os.path.exists(p.target):
+                    file_store.exportFile(FileID.unpack(p.resolved[7:]), "file://" + p.target)
+                elif p.type == "Directory" and not os.path.exists(p.target):
+                    os.makedirs(p.target, 0o0755)
+                elif p.type == "CreateFile":
+                    with open(p.target, "wb") as n:
+                        n.write(p.resolved.encode("utf-8"))
 
     def _check_adjust(f: dict) -> dict:
-        f["location"] = schema_salad.ref_resolver.file_uri(
-            pm.mapper(f["location"])[1])
+        f["location"] = schema_salad.ref_resolver.file_uri(pm.mapper(f["location"])[1])
+
         if "contents" in f:
             del f["contents"]
         return f
@@ -800,10 +841,11 @@ class CWLJob(Job):
                 debug=False,
                 js_console=False,
                 force_docker_pull=False,
-                loadListing='',
+                loadListing=determine_load_listing(tool),
                 outdir='',
                 tmpdir='/tmp',  # TODO: use actual defaults here
-                stagedir='/var/lib/cwl'  # TODO: use actual defaults here
+                stagedir='/var/lib/cwl',  # TODO: use actual defaults here
+                cwlVersion=self.cwltool.metadata['cwlVersion']
             )
 
         req = tool.evalResources(self.builder, runtime_context)
@@ -876,7 +918,19 @@ class CWLJob(Job):
         return required_env_vars
 
     def run(self, file_store: AbstractFileStore) -> Any:
+        
+        # Adjust cwltool's logging to conform to Toil's settings.
+        # We need to make sure this happens in every worker process before we
+        # do CWL things.
+        cwllogger.removeHandler(defaultStreamHandler)
+        cwllogger.setLevel(logger.getEffectiveLevel())
+        
+        
         cwljob = resolve_dict_w_promises(self.cwljob)
+
+        if self.conditional.is_false(cwljob):
+            return self.conditional.skipped_outputs()
+
         fill_in_defaults(
             self.step_inputs,
             cwljob,
@@ -884,9 +938,6 @@ class CWLJob(Job):
 
         required_env_vars = self.populate_env_vars(cwljob)
 
-        if self.conditional.is_false(cwljob):
-            return self.conditional.skipped_outputs()
-
         immobile_cwljob_dict = copy.deepcopy(cwljob)
         for inp_id in immobile_cwljob_dict.keys():
             found = False
@@ -926,7 +977,7 @@ class CWLJob(Job):
 
         process_uuid = uuid.uuid4()
         started_at = datetime.datetime.now()
-        # Run the tool
+
         output, status = cwltool.executors.SingleJobExecutor().execute(
             process=self.cwltool,
             job_order_object=cwljob,
@@ -939,6 +990,8 @@ class CWLJob(Job):
         adjustDirObjs(output, functools.partial(
             get_listing, cwltool.stdfsaccess.StdFsAccess(outdir),
             recursive=True))
+            
+        adjustDirObjs(output, prepareDirectoryForUpload)
 
         # write the outputs into the jobstore
         adjustFileObjs(output, functools.partial(
@@ -1355,10 +1408,48 @@ def remove_unprocessed_secondary_files(unfiltered_secondary_files: dict) -> list
     return final_secondary_files
 
 
+def determine_load_listing(tool: ToilCommandLineTool):
+    """
+    Determines the directory.listing feature in CWL.
+
+    In CWL, any input directory can have a DIRECTORY_NAME.listing (where DIRECTORY_NAME is any variable name) set to
+    one of the following three options:
+
+        no_listing: DIRECTORY_NAME.listing will be undefined.
+            e.g. inputs.DIRECTORY_NAME.listing == unspecified
+
+        shallow_listing: DIRECTORY_NAME.listing will return a list one level deep of DIRECTORY_NAME's contents.
+            e.g. inputs.DIRECTORY_NAME.listing == [items in directory]
+                 inputs.DIRECTORY_NAME.listing[0].listing == undefined
+                 inputs.DIRECTORY_NAME.listing.length == # of items in directory
+
+        deep_listing: DIRECTORY_NAME.listing will return a list of the entire contents of DIRECTORY_NAME.
+            e.g. inputs.DIRECTORY_NAME.listing == [items in directory]
+                 inputs.DIRECTORY_NAME.listing[0].listing == [items in subdirectory if it exists and is the first item listed]
+                 inputs.DIRECTORY_NAME.listing.length == # of items in directory
+
+    See: https://www.commonwl.org/v1.1/CommandLineTool.html#LoadListingRequirement
+         https://www.commonwl.org/v1.1/CommandLineTool.html#LoadListingEnum
+
+    DIRECTORY_NAME.listing should be determined first from loadListing.
+    If that's not specified, from LoadListingRequirement.
+    Else, default to "no_listing" if unspecified.
+
+    :param tool: ToilCommandLineTool
+    :return str: One of 'no_listing', 'shallow_listing', or 'deep_listing'.
+    """
+    load_listing_req, _ = tool.get_requirement("LoadListingRequirement")
+    load_listing_tool_req = load_listing_req.get("loadListing", "no_listing") if load_listing_req else "no_listing"
+    load_listing = tool.tool.get("loadListing") or load_listing_tool_req
+    return load_listing
+
+
 def main(args: Union[List[str]] = None, stdout: TextIO = sys.stdout) -> int:
     """Main method for toil-cwl-runner."""
+    # Remove cwltool logger's stream handler so it uses Toil's
     cwllogger.removeHandler(defaultStreamHandler)
     config = Config()
+    config.disableChaining = True
     config.cwl = True
     parser = argparse.ArgumentParser()
     addOptions(parser, config)
@@ -1526,6 +1617,8 @@ def main(args: Union[List[str]] = None, stdout: TextIO = sys.stdout) -> int:
     use_container = not options.no_container
 
     if options.logLevel:
+        # Make sure cwltool uses Toil's log level.
+        # Applies only on the leader.
         cwllogger.setLevel(options.logLevel)
 
     outdir = os.path.abspath(options.outdir)
@@ -1623,8 +1716,7 @@ def main(args: Union[List[str]] = None, stdout: TextIO = sys.stdout) -> int:
                 visit_class(inner_tool, ("File", "Directory"), path_to_loc)
                 visit_class(inner_tool, ("File",), functools.partial(add_sizes, fs_access))
                 normalizeFilesDirs(inner_tool)
-                adjustDirObjs(inner_tool, functools.partial(
-                    get_listing, fs_access, recursive=True))
+
                 adjustFileObjs(inner_tool, functools.partial(
                     uploadFile, toil.importFile, fileindex, existing,
                     skip_broken=True))  # actually import files into the jobstore
@@ -1661,6 +1753,11 @@ def main(args: Union[List[str]] = None, stdout: TextIO = sys.stdout) -> int:
             # using cwltool's library because we need to resolve them before toil imports them into the filestore.
             # A second builder will be built in the job's run method when toil actually starts the cwl job.
             builder = tool._init_job(initialized_job_order, runtime_context)
+
+            # make sure this doesn't add listing items; if shallow_listing is selected, it will discover dirs one deep
+            # and then again later on (producing 2+ deep listings instead of only 1)
+            builder.loadListing = 'no_listing'
+
             builder.bind_input(
                     tool.inputs_record_schema,
                     initialized_job_order,
