@@ -33,11 +33,10 @@ import uuid
 import base64
 import hashlib
 import itertools
+import reprlib
 import urllib.parse
 import urllib.request, urllib.parse, urllib.error
 from io import BytesIO
-# Python 3 compatibility imports
-from six.moves import StringIO, reprlib
 
 from toil.lib.memoize import strict_bool
 from toil.lib.exceptions import panic
@@ -50,7 +49,7 @@ from boto.exception import SDBResponseError, S3ResponseError
 import botocore.session
 import botocore.credentials
 
-from toil.lib.compatibility import compat_bytes, compat_plain, USING_PYTHON2
+from toil.lib.compatibility import compat_bytes, compat_plain
 from toil.lib.misc import AtomicFileCreate
 from toil.fileStores import FileID
 from toil.jobStores.abstractJobStore import (AbstractJobStore,
@@ -69,7 +68,7 @@ from toil.jobStores.aws.utils import (SDBHelper,
                                       bucket_location_to_region,
                                       region_to_bucket_location, copyKeyMultipart,
                                       uploadFromPath, chunkedFileUpload, fileSizeAndTime)
-from toil.jobStores.utils import WritablePipe, ReadablePipe
+from toil.jobStores.utils import WritablePipe, ReadablePipe, ReadableTransformingPipe
 from toil.jobGraph import JobGraph
 import toil.lib.encryption as encryption
 
@@ -82,6 +81,11 @@ s3_boto3_resource = boto3_session.resource('s3')
 s3_boto3_client = boto3_session.client('s3')
 log = logging.getLogger(__name__)
 
+class ChecksumError(Exception):
+    """
+    Raised when a download from AWS does not contain the correct data.
+    """
+    pass
 
 class AWSJobStore(AbstractJobStore):
     """
@@ -420,7 +424,7 @@ class AWSJobStore(AbstractJobStore):
                 info.copyFrom(srcKey)
                 info.save()
             finally:
-                srcKey.bucket.connection.close()
+                srcKey.bucket.connection.close() 
             return FileID(info.fileID, size) if sharedFileName is None else None
         else:
             return super(AWSJobStore, self)._importFile(otherCls, url,
@@ -866,7 +870,8 @@ class AWSJobStore(AbstractJobStore):
             :param numContentChunks: the number of SDB domain attributes occupied by this files
 
             :type checksum: str|None
-            :param checksum: the sha1 checksum of the file, if available.
+            :param checksum: the checksum of the file, if available. Formatted
+            as <algorithm>$<lowercase hex hash>.
 
             inlined content. Note that an inlined empty string still occupies one chunk.
             """
@@ -1078,6 +1083,8 @@ class AWSJobStore(AbstractJobStore):
             if file_size <= self.maxInlinedSize():
                 with open(localFilePath, 'rb') as f:
                     self.content = f.read()
+                # Clear out any old checksum in case of overwrite
+                self.checksum = ''
             else:
                 headers = self._s3EncryptionHeaders()
                 self.checksum = self._get_file_checksum(localFilePath) if calculateChecksum else None
@@ -1085,14 +1092,69 @@ class AWSJobStore(AbstractJobStore):
                                               bucket=self.outer.filesBucket, fileID=compat_bytes(self.fileID),
                                               headers=headers)
 
-        def _get_file_checksum(self, localFilePath):
+        def _start_checksum(self, to_match=None, algorithm='sha1'):
+            """
+            Get a hasher that can be used with _update_checksum and
+            _finish_checksum.
+            
+            If to_match is set, it is a precomputed checksum which we expect
+            the result to match.
+            
+            The right way to compare checksums is to feed in the checksum to be
+            matched, so we can see its algorithm, instead of getting a new one
+            and comparing. If a checksum to match is fed in, _finish_checksum()
+            will raise a ChecksumError if it isn't matched.
+            """
+            
+            # If we have an expexted result it will go here.
+            expected = None
+            
+            if to_match is not None:
+                parts = to_match.split('$')
+                algorithm = parts[0]
+                expected = parts[1]
+            
+            wrapped = getattr(hashlib, algorithm)()
+            
+            log.debug('Starting %s checksum to match %s', algorithm, expected)
+            
+            return (algorithm, wrapped, expected)
+            
+        def _update_checksum(self, checksum_in_progress, data):
+            """
+            Update a checksum in progress from _start_checksum with new data.
+            """
+            log.debug('Updating checksum with %d bytes', len(data))
+            checksum_in_progress[1].update(data)
+        
+        def _finish_checksum(self, checksum_in_progress):
+            """
+            Complete a checksum in progress from _start_checksum and return the
+            checksum result string.
+            """
+            
+            result_hash = checksum_in_progress[1].hexdigest()
+            
+            log.debug('Completed checksum with hash %s vs. expected %s', result_hash, checksum_in_progress[2])
+            
+            if checksum_in_progress[2] is not None:
+                # We expected a particular hash
+                if result_hash != checksum_in_progress[2]:
+                    raise ChecksumError('Checksum mismatch. Expected: %s Actual: %s' %
+                        (checksum_in_progress[2], result_hash))
+                    
+            return '$'.join([checksum_in_progress[0], result_hash])
+            
+            
+        
+        def _get_file_checksum(self, localFilePath, to_match=None):
             with open(localFilePath, 'rb') as f:
-                checksum = hashlib.sha1()
+                hasher = self._start_checksum(to_match=to_match)
                 contents = f.read(1024 * 1024)
                 while contents != b'':
-                    checksum.update(contents)
+                    self._update_checksum(hasher, contents)
                     contents = f.read(1024 * 1024)
-                return 'sha1$%s' % checksum.hexdigest()
+                return self._finish_checksum(hasher)
 
         @contextmanager
         def uploadStream(self, multipart=True, allowInlining=True):
@@ -1111,12 +1173,20 @@ class AWSJobStore(AbstractJobStore):
 
             class MultiPartPipe(WritablePipe):
                 def readFrom(self, readable):
+                    # Get the first block of data we want to put
                     buf = readable.read(store.partSize)
                     assert isinstance(buf, bytes)
+                    
                     if allowInlining and len(buf) <= info.maxInlinedSize():
                         log.debug('Inlining content of %d bytes', len(buf))
                         info.content = buf
+                        # There will be no checksum
+                        info.checksum = ''
                     else:
+                        # We will compute a checksum
+                        hasher = info._start_checksum()
+                        info._update_checksum(hasher, buf)
+                    
                         headers = info._s3EncryptionHeaders()
                         for attempt in retry_s3():
                             with attempt:
@@ -1126,27 +1196,21 @@ class AWSJobStore(AbstractJobStore):
                                     headers=headers)
                         try:
                             for part_num in itertools.count():
-                                # There must be at least one part, even if the file is empty.
-                                if len(buf) == 0 and part_num > 0:
-                                    break
                                 for attempt in retry_s3():
                                     with attempt:
                                         log.debug('Uploading part %d of %d bytes', part_num + 1, len(buf))
-                                        if USING_PYTHON2:
-                                            upload.upload_part_from_file(fp=StringIO(buf),
-                                                                         # part numbers are 1-based
-                                                                         part_num=part_num + 1,
-                                                                         headers=headers)
-                                        else:
-                                            upload.upload_part_from_file(fp=BytesIO(buf),
-                                                                         # part numbers are 1-based
-                                                                         part_num=part_num + 1,
-                                                                         headers=headers)
-
-                                if len(buf) == 0:
-                                    break
+                                        upload.upload_part_from_file(fp=BytesIO(buf),
+                                                                     # part numbers are 1-based
+                                                                     part_num=part_num + 1,
+                                                                     headers=headers)
+                                
+                                # Get the next block of data we want to put
                                 buf = readable.read(info.outer.partSize)
                                 assert isinstance(buf, bytes)
+                                if len(buf) == 0:
+                                    # Don't allow any part other than the very first to be empty.
+                                    break
+                                info._update_checksum(hasher, buf)
                         except:
                             with panic(log=log):
                                 for attempt in retry_s3():
@@ -1157,6 +1221,9 @@ class AWSJobStore(AbstractJobStore):
                             while store._getBucketVersioning(store.filesBucket) != True:
                                 log.warning('Versioning does not appear to be enabled yet. Deferring multipart upload completion...')
                                 time.sleep(1)
+                                
+                            # Save the checksum
+                            info.checksum = info._finish_checksum(hasher)
 
                             for attempt in retry_s3():
                                 with attempt:
@@ -1186,7 +1253,14 @@ class AWSJobStore(AbstractJobStore):
                     if allowInlining and dataLength <= info.maxInlinedSize():
                         log.debug('Inlining content of %d bytes', len(buf))
                         info.content = buf
+                        # There will be no checksum
+                        info.checksum = ''
                     else:
+                        # We will compute a checksum
+                        hasher = info._start_checksum()
+                        info._update_checksum(hasher, buf)
+                        info.checksum = info._finish_checksum(hasher)
+                        
                         key = store.filesBucket.new_key(key_name=compat_bytes(info.fileID))
                         buf = BytesIO(buf)
                         headers = info._s3EncryptionHeaders()
@@ -1194,6 +1268,8 @@ class AWSJobStore(AbstractJobStore):
                         while store._getBucketVersioning(store.filesBucket) != True:
                             log.warning('Versioning does not appear to be enabled yet. Deferring single part upload...')
                             time.sleep(1)
+                            
+                        
 
                         for attempt in retry_s3():
                             with attempt:
@@ -1290,23 +1366,27 @@ class AWSJobStore(AbstractJobStore):
             elif self.version:
                 headers = self._s3EncryptionHeaders()
                 key = self.outer.filesBucket.get_key(compat_bytes(self.fileID), validate=False)
-                for attempt in retry_s3():
+                for attempt in retry_s3(predicate=lambda e: retryable_s3_errors(e) or isinstance(e, ChecksumError)):
                     with attempt:
                         with AtomicFileCreate(localFilePath) as tmpPath:
                             key.get_contents_to_filename(tmpPath,
                                                          version_id=self.version,
                                                          headers=headers)
-                if verifyChecksum and self.checksum:
-                    downloadedChecksum = self._get_file_checksum(localFilePath)
-                    if self.checksum != downloadedChecksum:
-                        raise AssertionError(
-                            'Checksums do not match for file %s. Expected: %s Actual: %s' % (
-                                localFilePath, self.checksum, downloadedChecksum))
+                                                         
+                        if verifyChecksum and self.checksum:
+                            try:
+                                # This automatically compares the result and matches the algorithm.
+                                self._get_file_checksum(localFilePath, self.checksum)
+                            except ChecksumError as e:
+                                # Annotate checksum mismatches with file name
+                                raise ChecksumError('Checksums do not match for file %s.' % localFilePath) from e
+                                # The error will get caught and result in a retry of the download until we run out of retries.
+                                # TODO: handle obviously truncated downloads by resuming instead.
             else:
                 assert False
 
         @contextmanager
-        def downloadStream(self):
+        def downloadStream(self, verifyChecksum=True):
             info = self
 
             class DownloadPipe(ReadablePipe):
@@ -1323,9 +1403,41 @@ class AWSJobStore(AbstractJobStore):
                                                          version_id=info.version)
                     else:
                         assert False
-
+            
+            class HashingPipe(ReadableTransformingPipe):
+                """
+                Class which checksums all the data read through it. If it
+                reaches EOF and the checksum isn't correct, raises
+                ChecksumError.
+                
+                Assumes info actually has a checksum.
+                """
+            
+                def transform(self, readable, writable):
+                    hasher = info._start_checksum(to_match=info.checksum)
+                    contents = readable.read(1024 * 1024)
+                    while contents != b'':
+                        info._update_checksum(hasher, contents)
+                        try:
+                            writable.write(contents)
+                        except BrokenPipeError:
+                            # Read was stopped early by user code.
+                            # Can't check the checksum.
+                            return
+                        contents = readable.read(1024 * 1024)
+                    # We reached EOF in the input.
+                    # Finish checksumming and verify.
+                    info._finish_checksum(hasher)
+                    # Now stop so EOF happens in the output.
+                    
             with DownloadPipe() as readable:
-                yield readable
+                if verifyChecksum and self.checksum:
+                    # Interpose a pipe to check the hash
+                    with HashingPipe(readable) as verified:
+                        yield verified
+                else:
+                    # No true checksum available, so don't hash
+                    yield readable
 
         def delete(self):
             store = self.outer
