@@ -16,11 +16,6 @@
 
 collect_ignore = []
 
-try:
-    import azure
-except ImportError:
-    collect_ignore.append("azureJobStore.py")
-
 try:
     import boto
 except ImportError:
