@@ -44,7 +44,7 @@ class WritablePipe(with_metaclass(ABCMeta, object)):
 
     More complicated, less illustrative tests:
 
-    Same as above, but provving that handles are closed:
+    Same as above, but proving that handles are closed:
 
     >>> x = os.dup(0); os.close(x)
     >>> class MyPipe(WritablePipe):
@@ -164,7 +164,7 @@ class ReadablePipe(with_metaclass(ABCMeta, object)):
 
     More complicated, less illustrative tests:
 
-    Same as above, but provving that handles are closed:
+    Same as above, but proving that handles are closed:
 
     >>> x = os.dup(0); os.close(x)
     >>> class MyPipe(ReadablePipe):
@@ -240,3 +240,57 @@ class ReadablePipe(with_metaclass(ABCMeta, object)):
                 # Only raise the child exception if there wasn't
                 # already an exception in the main thread
                 raise
+                
+class ReadableTransformingPipe(ReadablePipe):
+    """
+    A pipe which is constructed around a readable stream, and which provides a
+    context manager that gives a readable stream.
+    
+    Useful as a base class for pipes which have to transform or otherwise visit
+    bytes that flow through them, instead of just consuming or producing data.
+    
+    Clients should subclass it and implement :meth:`.transform`, like so:
+    
+    >>> import sys, shutil
+    >>> class MyPipe(ReadableTransformingPipe):
+    ...     def transform(self, readable, writable):
+    ...         writable.write(readable.read().decode('utf-8').upper().encode('utf-8'))
+    >>> class SourcePipe(ReadablePipe):
+    ...     def writeTo(self, writable):
+    ...         writable.write('Hello, world!\\n'.encode('utf-8'))
+    >>> with SourcePipe() as source:
+    ...     with MyPipe(source) as transformed:
+    ...         shutil.copyfileobj(codecs.getreader('utf-8')(transformed), sys.stdout)
+    HELLO, WORLD!
+    
+    The :meth:`.transform` method runs in its own thread, and should move data
+    chunk by chunk instead of all at once. It should finish normally if it
+    encounters either an EOF on the readable, or a :class:`BrokenPipeError` on
+    the writable. This means tat it should make sure to actually catch a
+    :class:`BrokenPipeError` when writing.
+    
+    See also: :class:`toil.lib.misc.WriteWatchingStream`.
+    
+    """
+    
+    def __init__(self, source):
+        super(ReadableTransformingPipe, self).__init__()
+        self.source = source
+        
+    @abstractmethod
+    def transform(self, readable, writable):
+        """
+        Implement this method to ship data through the pipe.
+
+        :param file readable: the input stream file object to transform.
+
+        :param file writable: the file object representing the writable end of the pipe. Do not
+        explicitly invoke the close() method of the object, that will be done automatically.
+        """
+        raise NotImplementedError()
+    
+    def writeTo(self, writable):
+        self.transform(self.source, writable)
+    
+    
+
