@@ -35,6 +35,9 @@ from toil.test import (ToilTest, needs_cwl, slow, needs_docker, needs_lsf,
                        needs_torque)
 
 
+CONFORMANCE_TEST_TIMEOUT = 3600
+
+
 @needs_cwl
 class CWLv10Test(ToilTest):
     def setUp(self):
@@ -46,7 +49,7 @@ class CWLv10Test(ToilTest):
         self.workDir = os.path.join(self.cwlSpec, 'v1.0')
         # The latest cwl git commit hash from https://github.com/common-workflow-language/common-workflow-language.
         # Update it to get the latest tests.
-        testhash = '40fcfc01812046f012acf5153cc955ee848e69e3' # Date:   Tue Jan 21 07:36:37 2020 +0100
+        testhash = '40fcfc01812046f012acf5153cc955ee848e69e3'  # Date:   Tue Jan 21 07:36:37 2020 +0100
         url = 'https://github.com/common-workflow-language/common-workflow-language/archive/%s.zip' % testhash
         if not os.path.exists(self.cwlSpec):
             urlretrieve(url, 'spec.zip')
@@ -170,14 +173,12 @@ class CWLv10Test(ToilTest):
             pass
 
     @slow
-    @pytest.mark.timeout(2400)
-    # Cannot work until we fix https://github.com/DataBiosphere/toil/issues/2801
-    @pytest.mark.xfail
+    @pytest.mark.timeout(CONFORMANCE_TEST_TIMEOUT)
     def test_run_conformance_with_caching(self):
         self.test_run_conformance(caching=True)
 
     @slow
-    @pytest.mark.timeout(2400)
+    @pytest.mark.timeout(CONFORMANCE_TEST_TIMEOUT)
     def test_run_conformance(self, batchSystem=None, caching=False):
         try:
             cmd = ['cwltest', '--tool', 'toil-cwl-runner', '--test=conformance_test_v1.0.yaml',
@@ -206,48 +207,36 @@ class CWLv10Test(ToilTest):
     @slow
     @needs_lsf
     @unittest.skip
-    # Cannot work until we fix https://github.com/DataBiosphere/toil/issues/2801
-    @pytest.mark.xfail
     def test_lsf_cwl_conformance_with_caching(self):
         return self.test_run_conformance(batchSystem="LSF", caching=True)
 
     @slow
     @needs_slurm
     @unittest.skip
-    # Cannot work until we fix https://github.com/DataBiosphere/toil/issues/2801
-    @pytest.mark.xfail
     def test_slurm_cwl_conformance_with_caching(self):
         return self.test_run_conformance(batchSystem="Slurm", caching=True)
 
     @slow
     @needs_torque
     @unittest.skip
-    # Cannot work until we fix https://github.com/DataBiosphere/toil/issues/2801
-    @pytest.mark.xfail
     def test_torque_cwl_conformance_with_caching(self):
         return self.test_run_conformance(batchSystem="Torque", caching=True)
 
     @slow
     @needs_gridengine
     @unittest.skip
-    # Cannot work until we fix https://github.com/DataBiosphere/toil/issues/2801
-    @pytest.mark.xfail
     def test_gridengine_cwl_conformance_with_caching(self):
         return self.test_run_conformance(batchSystem="gridEngine", caching=True)
 
     @slow
     @needs_mesos
     @unittest.skip
-    # Cannot work until we fix https://github.com/DataBiosphere/toil/issues/2801
-    @pytest.mark.xfail
     def test_mesos_cwl_conformance_with_caching(self):
         return self.test_run_conformance(batchSystem="mesos", caching=True)
 
     @slow
     @needs_parasol
     @unittest.skip
-    # Cannot work until we fix https://github.com/DataBiosphere/toil/issues/2801
-    @pytest.mark.xfail
     def test_parasol_cwl_conformance_with_caching(self):
         return self.test_run_conformance(batchSystem="parasol", caching=True)
 
@@ -350,18 +339,18 @@ class CWLv11Test(ToilTest):
         unittest.TestCase.tearDown(self)
 
     @slow
-    @pytest.mark.timeout(2400)
+    @pytest.mark.timeout(CONFORMANCE_TEST_TIMEOUT)
     # Cannot work until we fix https://github.com/DataBiosphere/toil/issues/2801
     @pytest.mark.xfail
     def test_run_conformance_with_caching(self):
         self.test_run_conformance(caching=True)
 
     @slow
-    @pytest.mark.timeout(2400)
+    @pytest.mark.timeout(CONFORMANCE_TEST_TIMEOUT)
     def test_run_conformance(self, batchSystem=None, caching=False):
         try:
-            # TODO: we do not currently pass tests: 213, 236, 242, 243, 244, 245, 246, 249
-            selected_tests = '1-212,214-235,237-241,247-248,250-253'
+            # TODO: we do not currently pass tests: 213, 236
+            selected_tests = '1-212,214-235,237-253'
             cmd = [f'cwltest',
                    f'--tool=toil-cwl-runner',
                    f'--test={self.test_yaml}',
@@ -401,7 +390,7 @@ class CWLv12Test(ToilTest):
         cls.test_yaml = os.path.join(cls.cwlSpec, 'conformance_tests.yaml')
         # TODO: Use a commit zip in case someone decides to rewrite master's history?
         url = 'https://github.com/common-workflow-language/cwl-v1.2.git'
-        commit = 'fca122ef126d03da8c8091111ef5d0cf75763382'
+        commit = '936961d1c1c5c4ea13df253b1316a264e48df740'
         p = subprocess.Popen(f'git clone {url} {cls.cwlSpec} && cd {cls.cwlSpec} && git checkout {commit}', shell=True)
         p.communicate()
 
@@ -412,18 +401,18 @@ class CWLv12Test(ToilTest):
         unittest.TestCase.tearDown(self)
 
     @slow
-    @pytest.mark.timeout(2400)
+    @pytest.mark.timeout(CONFORMANCE_TEST_TIMEOUT)
     # Cannot work until we fix https://github.com/DataBiosphere/toil/issues/2801
     @pytest.mark.xfail
     def test_run_conformance_with_caching(self):
         self.test_run_conformance(caching=True)
 
     @slow
-    @pytest.mark.timeout(2400)
+    @pytest.mark.timeout(CONFORMANCE_TEST_TIMEOUT)
     def test_run_conformance(self, batchSystem=None, caching=False):
         try:
-            # TODO: we do not currently pass tests: 213, 236, 242, 243, 244, 245, 246, 249
-            selected_tests = '1-212,214-235,237-241,247-248,250-276'
+            # TODO: we do not currently pass tests: 214, 237 (offset from other versions)
+            selected_tests = '1-213,215-236,238-276'
             cmd = [f'cwltest',
                    f'--tool=toil-cwl-runner',
                    f'--test={self.test_yaml}',
