@@ -8,7 +8,7 @@ import shutil
 pkg_root = os.path.abspath(os.path.join(os.path.dirname(__file__), '..'))  # noqa
 sys.path.insert(0, pkg_root)  # noqa
 
-from toil import subprocess
+import subprocess
 from toil.test import ToilTest
 from toil.test import needs_cwl
 from toil.version import python
