@@ -11,10 +11,10 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-from builtins import str
 import logging
 import os
-from toil import subprocess
+
+import subprocess
 from toil.version import exactPython
 from abc import abstractmethod
 
