@@ -5,13 +5,16 @@ import os
 import tempfile
 from functools import partial
 
-from toil.test import ToilTest
+from toil.lib.threading import cpu_count
+from toil.test import ToilTest, travis_test
 
 
 class SystemTest(ToilTest):
     """
     Test various assumptions about the operating system's behavior
     """
+    
+    @travis_test
     def testAtomicityOfNonEmptyDirectoryRenames(self):
         for _ in range(100):
             parent = self._createTempDir(purpose='parent')
@@ -19,7 +22,7 @@ class SystemTest(ToilTest):
             # Use processes (as opposed to threads) to prevent GIL from ordering things artificially
             pool = multiprocessing.Pool()
             try:
-                numTasks = multiprocessing.cpu_count() * 10
+                numTasks = cpu_count() * 10
                 grandChildIds = pool.map_async(
                     func=partial(_testAtomicityOfNonEmptyDirectoryRenamesTask, parent, child),
                     iterable=list(range(numTasks)))
