@@ -1,7 +1,7 @@
 from toil.job import Job
 from toil.common import Toil
+import subprocess
 from toil.version import python
-from toil import subprocess
 import os
 import logging
 
