@@ -29,7 +29,7 @@ import psutil
 
 import toil
 import toil.test.sort.sort
-from toil import subprocess
+import subprocess
 from toil import resolveEntryPoint
 from toil.job import Job
 from toil.utils.toilStatus import ToilStatus
@@ -40,6 +40,7 @@ from toil.utils.toilStats import getStats, processData
 from toil.common import Toil, Config
 from toil.provisioners import clusterFactory
 from toil.version import python
+from mock import patch
 
 logger = logging.getLogger(__name__)
 
@@ -182,7 +183,7 @@ class UtilsTest(ToilTest):
 
             # `toil rsync-cluster`
             # Testing special characters - string.punctuation
-            fname = '!"#$%&\'()*+,-.;<=>:\ ?@[\\\\]^_`{|}~'
+            fname = r'!"#$%&\'()*+,-.;<=>:\ ?@[\\]^_`{|}~'
             testData = os.urandom(3 * (10**6))
             with tempfile.NamedTemporaryFile(suffix=fname) as tmpFile:
                 relpath = os.path.basename(tmpFile.name)
@@ -387,6 +388,23 @@ class UtilsTest(ToilTest):
         wf.wait()
         self.check_status('COMPLETED', status_fn=ToilStatus.getStatus)
 
+    @needs_cwl
+    @patch('builtins.print')
+    def testPrintJobLog(self, mock_print):
+        """Test that ToilStatus.printJobLog() reads the log from a failed command without error."""
+        # Run a workflow that will always fail
+        cmd = ['toil-cwl-runner', '--jobStore', self.toilDir, '--clean=never',
+               'src/toil/test/cwl/alwaysfails.cwl', '--message', 'Testing']
+        wf = subprocess.Popen(cmd)
+        wf.wait()
+        # print log and check output
+        status = ToilStatus(self.toilDir)
+        status.printJobLog()
+        
+        # Make sure it printed some kind of complaint about the missing command.
+        args, kwargs = mock_print.call_args
+        self.assertIn('invalidcommand', args[0])
+
 
 def printUnicodeCharacter():
     # We want to get a unicode character to stdout but we can't print it directly because of
